"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

# -*- coding: utf-8 -*-
from flask import current_app
from constants import *


def get_info(cluster_obj):

    cluster_obj.get_updates()
    info = dict()
    info['nodes'] = list(cluster_obj.get_node_list())
    info['namespaces'] = list(cluster_obj.get_namespaces())
    info['cluster_status'] = cluster_obj.get_status()
    info['indexes'] = cluster_obj.get_index_list_by_namespace()
    info['build'] = cluster_obj.get_build_no()
    info['update_interval'] = current_app.config.get("UPDATE_INTERVAL")
    info['different_cluster_nodes'] = cluster_obj.get_different_cluster_nodes()
    return info


def get_basic_info(cluster_obj):

    cluster_obj.get_updates()
    info = dict()
    info['nodes'] = list(cluster_obj.get_node_list())
    info['off_nodes'] = cluster_obj.get_off_nodes()
    info['memory'] = cluster_obj.get_memory_usage()
    info['disk'] = cluster_obj.get_disk_usage()
    info['namespaces'] = list(cluster_obj.get_namespaces())
    info['cluster_status'] = cluster_obj.get_status()
    info['build'] = cluster_obj.get_build_no()
    info['update_interval'] = current_app.config.get("UPDATE_INTERVAL")
    info['build_details'] = cluster_obj.get_build_no_of_cluster_nodes()
    return info


def get_clusters(clusters):

    clusters_info = dict(clusters=[{"id": cluster_id, "nodes": cluster_obj.get_node_list()}
        for cluster_id, cluster_obj in list(clusters.get_clusters().items())])
    return clusters_info


def get_throughput(cluster_obj, node_addresses):

    if node_addresses is None:
        node_addresses = cluster_obj.get_node_list()
    else:
        node_addresses = node_addresses.split(',')

    throughput = dict(read_tps={}, write_tps={}, cluster_status=cluster_obj.get_status())
    for node_address in node_addresses:
        node = cluster_obj.get_node(node_address)
        if node is None or node.get_status() == "off" or not node.is_valid():
            continue
        #throughput["read_tps"][node_address] = node.get_statistic("read_tps")
        #throughput["write_tps"][node_address] = node.get_statistic("write_tps")
        throughput["read_tps"][node_address] = node.get_read_tps()
        throughput["write_tps"][node_address] = node.get_write_tps()
    return throughput


def get_throughput_history(cluster_obj, node_addresses, start_time=None, end_time=None):

    if node_addresses is None:
        node_addresses = cluster_obj.get_node_list()
    else:
        node_addresses = node_addresses.split(',')

    throughput_history = dict(read_tps={}, write_tps={}, cluster_status=cluster_obj.get_status())
    for node_address in node_addresses:
        node = cluster_obj.get_node(node_address)
        if node is None:
            continue
        throughput_history["read_tps"][node_address] = list(node.get_read_tps_history(start_time, end_time))
        throughput_history["write_tps"][node_address] = list(node.get_write_tps_history(start_time, end_time))
    return throughput_history


def get_indexes_by_namespace(cluster_obj, namespace_name):

    result = {"cluster_status": "on"}

    indexes = cluster_obj.get_indexes_by_namespaces(namespace_name)

    if indexes == UNABLE_TO_CONNECT:
        result["cluster_status"] = "off"
        return result

    result["indexes"] = indexes
    return result


def get_sets_by_namespace(cluster_obj, namespace_name):

    result = {"cluster_status": "on"}

    sets = cluster_obj.get_sets_info_by_namespaces(namespace_name)
    if sets == UNABLE_TO_CONNECT:
        result["cluster_status"] = "off"
        return result

    result["sets"] = sets
    return result


def add_node(cluster_obj, node_address):

    return cluster_obj.add_existing_node(node_address)


def remove_node(cluster_obj, node_address):

    return cluster_obj.remove_node(node_address)
